@@ -1,4 +1,5 @@
-# Copyright (C) 2003, 2004  Free Software Foundation, Inc.
+# Copyright (C) 2003, 2004, 2005, 2006, 2008, 2009, 2010 Free Software
+# Foundation, Inc.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,9 +12,7 @@
 # GNU General Public License for more details.
 
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-# 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 package Automake::Variable;
 use strict;
@@ -73,7 +72,7 @@ Automake::Variable - support for variable definitions
   # $var->conditions->conds is a list of Automake::Condition.)
   my @conds = $var->conditions->conds
 
-  # Accessing to the definition in Condition $cond.
+  # Access to the definition in Condition $cond.
   # $def is an Automake::VarDef.
   my $def = $var->def ($cond);
   if ($def)
@@ -130,18 +129,26 @@ non-object).
 
 =cut
 
-my $_VARIABLE_PATTERN = '^[.A-Za-z0-9_@]+' . "\$";
+my $_VARIABLE_CHARACTERS = '[.A-Za-z0-9_@]+';
+my $_VARIABLE_PATTERN = '^' . $_VARIABLE_CHARACTERS . "\$";
+my $_VARIABLE_RECURSIVE_PATTERN =
+    '^([.A-Za-z0-9_@]|\$[({]' . $_VARIABLE_CHARACTERS . '[})]?)+' . "\$";
 
 # The order in which variables should be output.  (May contain
 # duplicates -- only the first occurrence matters.)
 my @_var_order;
 
 # This keeps track of all variables defined by &_gen_varname.
-# $_gen_varname{$base} is a hash for all variable defined with
-# prefix `$base'.  Values stored this this hash are the variable names.
+# $_gen_varname{$base} is a hash for all variables defined with
+# prefix `$base'.  Values stored in this hash are the variable names.
 # Keys have the form "(COND1)VAL1(COND2)VAL2..." where VAL1 and VAL2
 # are the values of the variable for condition COND1 and COND2.
 my %_gen_varname = ();
+# $_gen_varname_n{$base} is the number of variables generated by
+# _gen_varname() for $base.  This is not the same as keys
+# %{$_gen_varname{$base}} because %_gen_varname may also contain
+# variables not generated by _gen_varname.
+my %_gen_varname_n = ();
 
 # Declare the macros that define known variables, so we can
 # hint the user if she try to use one of these variables.
@@ -155,7 +162,7 @@ my %_am_macro_for_var =
    EMACS => 'AM_PATH_LISPDIR',
    GCJ => 'AM_PROG_GCJ',
    LEX => 'AM_PROG_LEX',
-   LIBTOOL => 'AC_PROG_LIBTOOL',
+   LIBTOOL => 'LT_INIT',
    lispdir => 'AM_PATH_LISPDIR',
    pkgpyexecdir => 'AM_PATH_PYTHON',
    pkgpythondir => 'AM_PATH_PYTHON',
@@ -175,25 +182,30 @@ my %_ac_macro_for_var =
    CXXFLAGS => 'AC_PROG_CXX',
    F77 => 'AC_PROG_F77',
    F77FLAGS => 'AC_PROG_F77',
+   FC => 'AC_PROG_FC',
+   FCFLAGS => 'AC_PROG_FC',
+   OBJC => 'AC_PROG_OBJC',
+   OBJCFLAGS => 'AC_PROG_OBJC',
    RANLIB => 'AC_PROG_RANLIB',
+   UPC => 'AM_PROG_UPC',
+   UPCFLAGS => 'AM_PROG_UPC',
    YACC => 'AC_PROG_YACC',
    );
 
 # The name of the configure.ac file.
-my $configure_ac = find_configure_ac;
+my $configure_ac;
 
-# Variables that can be overriden without complaint from -Woverride
+# Variables that can be overridden without complaint from -Woverride
 my %_silent_variable_override =
   (AM_MAKEINFOHTMLFLAGS => 1,
    AR => 1,
    ARFLAGS => 1,
    DEJATOOL => 1,
-   JAVAC => 1);
+   JAVAC => 1,
+   JAVAROOT => 1);
 
-# This hash records helper variables used to implement conditional '+='.
-# Keys have the form "VAR:CONDITIONS".  The value associated to a key is
-# the named of the helper variable used to append to VAR in CONDITIONS.
-my %_appendvar = ();
+# Count of helper variables used to implement conditional '+='.
+my $_appendvar;
 
 # Each call to C<Automake::Variable::traverse_recursively> gets an
 # unique label. This is used to detect recursively defined variables.
@@ -292,17 +304,33 @@ sub hook ($$)
   $_hooks{$var} = $fun;
 }
 
-=item C<variables>
+=item C<variables ([$suffix])>
 
 Returns the list of all L<Automake::Variable> instances.  (I.e., all
-variables defined so far.)
+variables defined so far.)  If C<$suffix> is supplied, return only
+the L<Automake::Variable> instances that ends with C<_$suffix>.
 
 =cut
 
-use vars '%_variable_dict';
-sub variables ()
+use vars '%_variable_dict', '%_primary_dict';
+sub variables (;$)
 {
-  return values %_variable_dict;
+  my ($suffix) = @_;
+  if ($suffix)
+    {
+      if (exists $_primary_dict{$suffix})
+	{
+	  return values %{$_primary_dict{$suffix}};
+	}
+      else
+	{
+	  return ();
+	}
+    }
+  else
+    {
+      return values %_variable_dict;
+    }
 }
 
 =item C<Automake::Variable::reset>
@@ -315,16 +343,18 @@ other internal data.
 sub reset ()
 {
   %_variable_dict = ();
-  %_appendvar = ();
+  %_primary_dict = ();
+  $_appendvar = 0;
   @_var_order = ();
   %_gen_varname = ();
+  %_gen_varname_n = ();
   $_traversal = 0;
 }
 
 =item C<var ($varname)>
 
 Return the C<Automake::Variable> object for the variable
-named C<$varname> if defined.   Return 0 otherwise.
+named C<$varname> if defined.  Return 0 otherwise.
 
 =cut
 
@@ -416,7 +446,12 @@ sub _new ($$)
   my ($class, $name) = @_;
   my $self = Automake::Item::new ($class, $name);
   $self->{'scanned'} = 0;
+  $self->{'last-append'} = []; # helper variable for last conditional append.
   $_variable_dict{$name} = $self;
+  if ($name =~ /_([[:alnum:]]+)$/)
+    {
+      $_primary_dict{$1}{$name} = $self;
+    }
   return $self;
 }
 
@@ -432,8 +467,9 @@ sub _check_ambiguous_condition ($$$)
   my $var = $self->name;
   my ($message, $ambig_cond) = $self->conditions->ambiguous_p ($var, $cond);
 
-  # We allow silent variables to be overridden silently.
-  my $def = $self->def ($cond);
+  # We allow silent variables to be overridden silently,
+  # by either silent or non-silent variables.
+  my $def = $self->def ($ambig_cond);
   if ($message && !($def && $def->pretty == VAR_SILENT))
     {
       msg 'syntax', $where, "$message ...", partial => 1;
@@ -620,7 +656,7 @@ sub value_as_list_recursive ($;%)
      # Collect results.
      sub {
        my ($var, $parent_cond, @allresults) = @_;
-       return map { my ($cond, @vals) = @$_; return @vals } @allresults;
+       return map { my ($cond, @vals) = @$_; @vals } @allresults;
      },
      %options);
 }
@@ -710,7 +746,7 @@ sub scan_variable_expansions ($)
   while ($text =~ /(?<!\$)\$(?:\{([^\}]*)\}|\(([^\)]*)\))/g)
     {
       my $var = $1 || $2;
-      # The occurence may look like $(string1[:subst1=[subst2]]) but
+      # The occurrence may look like $(string1[:subst1=[subst2]]) but
       # we want only `string1'.
       $var =~ s/:[^:=]*=[^=]*$//;
       push @result, $var;
@@ -740,8 +776,17 @@ sub check_variable_expansions ($$)
 	  # Mention this in the diagnostic.
 	  my $gnuext = "";
 	  $gnuext = "\n(probably a GNU make extension)" if $var =~ / /;
-	  msg ('portability', $where,
-	       "$var: non-POSIX variable name$gnuext");
+	  # Accept recursive variable expansions if so desired
+	  # (we hope they are rather portable in practice).
+	  if ($var =~ /$_VARIABLE_RECURSIVE_PATTERN/o)
+	    {
+	      msg ('portability-recursive', $where,
+		   "$var: non-POSIX recursive variable expansion$gnuext");
+	    }
+	  else
+	    {
+	      msg ('portability', $where, "$var: non-POSIX variable name$gnuext");
+	    }
 	}
     }
 }
@@ -756,7 +801,7 @@ C<$varname>: the name of the variable being defined.
 
 C<$owner>: owner of the variable (one of C<VAR_MAKEFILE>,
 C<VAR_CONFIGURE>, or C<VAR_AUTOMAKE>, defined by L<Automake::VarDef>).
-Variables can be overriden, provided the new owner is not weaker
+Variables can be overridden, provided the new owner is not weaker
 (C<VAR_AUTOMAKE> < C<VAR_CONFIGURE> < C<VAR_MAKEFILE>).
 
 C<$type>: the type of the assignment (C<''> for C<FOO = bar>,
@@ -788,7 +833,7 @@ sub define ($$$$$$$$)
   my ($var, $owner, $type, $cond, $value, $comment, $where, $pretty) = @_;
 
   prog_error "$cond is not a reference"
-    unless ref $where;
+    unless ref $cond;
 
   prog_error "$where is not a reference"
     unless ref $where;
@@ -868,6 +913,7 @@ sub define ($$$$$$$$)
   if ($type eq '+' && ! $new_var)
     {
       $def->append ($value, $comment);
+      $self->{'last-append'} = [];
 
       # Only increase owners.  A VAR_CONFIGURE variable augmented in a
       # Makefile.am becomes a VAR_MAKEFILE variable.
@@ -900,32 +946,39 @@ sub define ($$$$$$$$)
       #     @COND_TRUE@FOO = foo1 bar
       #     @COND_FALSE@FOO = foo2 bar
 
+      my $lastappend = [];
       # Do we need an helper variable?
       if ($cond != TRUE)
         {
-	    # Does the helper variable already exists?
-	    my $key = "$var:" . $cond->string;
-	    if (exists $_appendvar{$key})
-	      {
-		# Yes, let's simply append to it.
-		$var = $_appendvar{$key};
-		$owner = VAR_AUTOMAKE;
-		$self = var ($var);
-		$def = $self->rdef ($cond);
-		$new_var = 0;
-	      }
-	    else
-	      {
-		# No, create it.
-		my $num = 1 + keys (%_appendvar);
-		my $hvar = "am__append_$num";
-		$_appendvar{$key} = $hvar;
-		&define ($hvar, VAR_AUTOMAKE, '+',
-			 $cond, $value, $comment, $where, $pretty);
-		# Now HVAR is to be added to VAR.
-		$comment = '';
-		$value = "\$($hvar)";
-	      }
+	  # Can we reuse the helper variable created for the previous
+	  # append?  (We cannot reuse older helper variables because
+	  # we must preserve the order of items appended to the
+	  # variable.)
+	  my $condstr = $cond->string;
+	  my $key = "$var:$condstr";
+	  my ($appendvar, $appendvarcond) = @{$self->{'last-append'}};
+	  if ($appendvar && $condstr eq $appendvarcond)
+	    {
+	      # Yes, let's simply append to it.
+	      $var = $appendvar;
+	      $owner = VAR_AUTOMAKE;
+	      $self = var ($var);
+	      $def = $self->rdef ($cond);
+	      $new_var = 0;
+	    }
+	  else
+	    {
+	      # No, create it.
+	      my $num = ++$_appendvar;
+	      my $hvar = "am__append_$num";
+	      $lastappend = [$hvar, $condstr];
+	      &define ($hvar, VAR_AUTOMAKE, '+',
+		       $cond, $value, $comment, $where, $pretty);
+
+	      # Now HVAR is to be added to VAR.
+	      $comment = '';
+	      $value = "\$($hvar)";
+	    }
 	}
 
       # Add VALUE to all definitions of SELF.
@@ -956,6 +1009,7 @@ sub define ($$$$$$$$)
 		       $where, $pretty);
 	    }
 	}
+      $self->{'last-append'} = $lastappend;
     }
   # 3. first assignment (=, :=, or +=)
   else
@@ -1009,6 +1063,10 @@ sub variable_delete ($@)
 	  delete $_variable_dict{$var}{'defs'}{$cond};
 	}
     }
+  if ($var =~ /_([[:alnum:]]+)$/)
+    {
+      delete $_primary_dict{$1}{$var};
+    }
 }
 
 =item C<$str = variables_dump>
@@ -1062,7 +1120,7 @@ sub set_seen ($)
 
 Make sure that each supplied variable is defined in C<$cond>.
 Otherwise, issue a warning showing C<$reason> (C<$reason> should be
-the reason why these variable are required, for instance C<'option foo
+the reason why these variables are required, for instance C<'option foo
 used'>).  If we know which macro can define this variable, hint the
 user.  Return the number of undefined variables.
 
@@ -1074,6 +1132,9 @@ sub require_variables ($$$@)
   my $res = 0;
   $reason .= ' but ' unless $reason eq '';
 
+  $configure_ac = find_configure_ac
+    unless defined $configure_ac;
+
  VARIABLE:
   foreach my $var (@vars)
     {
@@ -1089,21 +1150,28 @@ sub require_variables ($$$@)
 	  next VARIABLE
 	    if $undef_cond->false;
 	  $text .= ("in the following conditions:\n  "
-		    . join ("\n  ", map { $_->human } $undef_cond->conds));
+		    . join ("\n  ", map { $_->human } $undef_cond->conds)
+		    . "\n");
 	}
 
       ++$res;
 
       if (exists $_am_macro_for_var{$var})
 	{
-	  $text .= "\nThe usual way to define `$var' is to add "
-	    . "`$_am_macro_for_var{$var}'\nto `$configure_ac' and "
-	    . "run `aclocal' and `autoconf' again.";
+	  my $mac = $_am_macro_for_var{$var};
+	  $text .= "  The usual way to define `$var' is to add "
+	    . "`$mac'\n  to `$configure_ac' and run `aclocal' and "
+	    . "`autoconf' again.";
+	  # aclocal will not warn about undefined macros unless it
+	  # starts with AM_.
+	  $text .= "\n  If `$mac' is in `$configure_ac', make sure\n"
+	    . "  its definition is in aclocal's search path."
+	    unless $mac =~ /^AM_/;
 	}
       elsif (exists $_ac_macro_for_var{$var})
 	{
-	  $text .= "\nThe usual way to define `$var' is to add "
-	    . "`$_ac_macro_for_var{$var}'\nto `$configure_ac' and "
+	  $text .= "  The usual way to define `$var' is to add "
+	    . "`$_ac_macro_for_var{$var}'\n  to `$configure_ac' and "
 	    . "run `autoconf' again.";
 	}
 
@@ -1137,7 +1205,7 @@ sub requires_variables ($$@)
 
 Get the C<TRUE> value of a variable, warn if the variable is
 conditionally defined.  C<$var> can be either a variable name
-or a C<Automake::Variable> instance (this allows to calls sucha
+or a C<Automake::Variable> instance (this allows calls such
 as C<$var-E<gt>variable_value>).
 
 =cut
@@ -1148,7 +1216,8 @@ sub variable_value ($)
     my $v = ref ($var) ? $var : var ($var);
     return () unless $v;
     $v->check_defined_unconditionally;
-    return $v->rdef (TRUE)->value;
+    my $d = $v->def (TRUE);
+    return $d ? $d->value : "";
 }
 
 =item C<$str = output_variables>
@@ -1189,7 +1258,7 @@ sub output_variables ()
   return $res;
 }
 
-=item C<$var-E<gt>traverse_recursively (&fun_item, &fun_collect, [cond_filter =E<gt> $cond_filter], [inner_expand =E<gt> 1])>
+=item C<$var-E<gt>traverse_recursively (&fun_item, &fun_collect, [cond_filter =E<gt> $cond_filter], [inner_expand =E<gt> 1], [skip_ac_subst =E<gt> 1])>
 
 Split the value of the Automake::Variable C<$var> on space, and
 traverse its components recursively.
@@ -1209,18 +1278,21 @@ following arguments:
                    traversing
    $val,        -- the item (i.e., filename) to process
    $cond,       -- the Condition for the $var definition we are
-                   examinating (ignoring the recursion context)
+                   examining (ignoring the recursion context)
    $full_cond)  -- the full Condition, taking into account
                    conditions inherited from parent variables
                    during recursion
 
-If C<inner_expand> is set, variable references occuring in filename
-(as in C<$(BASE).ext>) are expansed before the filename is passed to
+If C<inner_expand> is set, variable references occurring in filename
+(as in C<$(BASE).ext>) are expanded before the filename is passed to
 C<&fun_item>.
 
+If C<skip_ac_subst> is set, Autoconf @substitutions@ will be skipped,
+i.e., C<&fun_item> will never be called for them.
+
 C<&fun_item> may return a list of items, they will be passed to
-C<&fun_store> later on.  Define C<&fun_item> as C<undef> when it serve
-no purpose, this will speed things up.
+C<&fun_store> later on.  Define C<&fun_item> or @<&fun_store> as
+C<undef> when they serve no purpose.
 
 Once all items of a variable have been processed, the result (of the
 calls to C<&fun_items>, or of recursive traversals of subvariables)
@@ -1254,16 +1326,18 @@ sub traverse_recursively ($&&;%)
   my ($var, $fun_item, $fun_collect, %options) = @_;
   my $cond_filter = $options{'cond_filter'};
   my $inner_expand = $options{'inner_expand'};
+  my $skip_ac_subst = $options{'skip_ac_subst'};
   return $var->_do_recursive_traversal ($var,
 					$fun_item, $fun_collect,
-					$cond_filter, TRUE, $inner_expand)
+					$cond_filter, TRUE, $inner_expand,
+					$skip_ac_subst)
 }
 
 # The guts of Automake::Variable::traverse_recursively.
-sub _do_recursive_traversal ($$&&$$$)
+sub _do_recursive_traversal ($$&&$$$$)
 {
   my ($var, $parent, $fun_item, $fun_collect, $cond_filter, $parent_cond,
-      $inner_expand) = @_;
+      $inner_expand, $skip_ac_subst) = @_;
 
   $var->set_seen;
 
@@ -1330,7 +1404,8 @@ sub _do_recursive_traversal ($$&&$$$)
 							  $fun_collect,
 							  $cond_filter,
 							  $full_cond,
-							  $inner_expand);
+							  $inner_expand,
+							  $skip_ac_subst);
 	      push (@result, @res);
 
 	      pop @_substfroms;
@@ -1375,6 +1450,10 @@ sub _do_recursive_traversal ($$&&$$$)
 	      # We do not know any variable with this name.  Fall through
 	      # to filename processing.
 	    }
+	  elsif ($skip_ac_subst && $val =~ /^\@.+\@$/)
+	    {
+	      next;
+	    }
 
 	  if ($fun_item) # $var is a filename we must process
 	    {
@@ -1400,19 +1479,55 @@ sub _do_recursive_traversal ($$&&$$$)
   # is free to use the same variable several times in the same definition.
   $var->{'scanned'} = -1;
 
+  return ()
+    unless $fun_collect;
   # Make sure you update the doc of Automake::Variable::traverse_recursively
   # if you change the prototype of &fun_collect.
   return &$fun_collect ($var, $parent_cond, @allresults);
 }
 
-# $VARNAME
+# _hash_varname ($VAR)
+# --------------------
+# Compute the key associated $VAR in %_gen_varname.
+# See _gen_varname() below.
+sub _hash_varname ($)
+{
+  my ($var) = @_;
+  my $key = '';
+  foreach my $cond ($var->conditions->conds)
+    {
+      my @values = $var->value_as_list ($cond);
+      $key .= "($cond)@values";
+    }
+  return $key;
+}
+
+# _hash_values (@VALUES)
+# ----------------------
+# Hash @VALUES for %_gen_varname.  @VALUES should be a list
+# of pairs: ([$cond, @values], [$cond, @values], ...).
+# See _gen_varname() below.
+sub _hash_values (@)
+{
+  my $key = '';
+  foreach my $pair (@_)
+    {
+      my ($cond, @values) = @$pair;
+      $key .= "($cond)@values";
+    }
+  return $key;
+}
+# ($VARNAME, $GENERATED)
 # _gen_varname ($BASE, @DEFINITIONS)
-# ---------------------------------
+# ----------------------------------
 # Return a variable name starting with $BASE, that will be
 # used to store definitions @DEFINITIONS.
 # @DEFINITIONS is a list of pair [$COND, @OBJECTS].
 #
-# If we already have a $BASE-variable containing @DEFINITIONS, reuse it.
+# If we already have a $BASE-variable containing @DEFINITIONS, reuse
+# it and set $GENERATED to 0.  Otherwise construct a new name and set
+# $GENERATED to 1.
+#
 # This way, we avoid combinatorial explosion of the generated
 # variables.  Especially, in a Makefile such as:
 #
@@ -1443,19 +1558,17 @@ sub _do_recursive_traversal ($$&&$$$)
 sub _gen_varname ($@)
 {
   my $base = shift;
-  my $key = '';
-  foreach my $pair (@_)
-    {
-      my ($cond, @values) = @$pair;
-      $key .= "($cond)@values";
-    }
+  my $key = _hash_values @_;
 
-  return $_gen_varname{$base}{$key} if exists $_gen_varname{$base}{$key};
+  return ($_gen_varname{$base}{$key}, 0)
+    if exists $_gen_varname{$base}{$key};
 
-  my $num = 1 + keys (%{$_gen_varname{$base}});
+  my $num = 1 + ($_gen_varname_n{$base} || 0);
+  $_gen_varname_n{$base} = $num;
   my $name = "${base}_${num}";
   $_gen_varname{$base}{$key} = $name;
-  return $name;
+
+  return ($name, 1);
 }
 
 =item C<$resvar = transform_variable_recursively ($var, $resvar, $base, $nodefine, $where, &fun_item, [%options])>
@@ -1495,33 +1608,54 @@ sub transform_variable_recursively ($$$$$&;%)
 
   my $res = $var->traverse_recursively
     ($fun_item,
-     # The code that define the variable holding the result
+     # The code that defines the variable holding the result
      # of the recursive transformation of a subvariable.
      sub {
        my ($subvar, $parent_cond, @allresults) = @_;
+       # If no definition is required, return anything: the result is
+       # not expected to be used, only the side effect of $fun_item
+       # should matter.
+       return 'report-me' if $nodefine;
+       # Cache $subvar, so that we reuse it if @allresults is the same.
+       my $key = _hash_varname $subvar;
+       $_gen_varname{$base}{$key} = $subvar->name;
+
        # Find a name for the variable, unless this is the top-variable
        # for which we want to use $resvar.
-       my $varname =
-	 ($var != $subvar) ? _gen_varname ($base, @allresults) : $resvar;
-       # Define the variable if required.
-       unless ($nodefine)
+       my ($varname, $generated) =
+	 ($var != $subvar) ? _gen_varname ($base, @allresults) : ($resvar, 1);
+
+       # Define the variable if we are not reusing a previously
+       # defined variable.  At the top-level, we can also avoid redefining
+       # the variable if it already contains the same values.
+       if ($generated
+	   && !($varname eq $var->name && $key eq _hash_values @allresults))
 	 {
 	   # If the new variable is the source variable, we assume
 	   # we are trying to override a user variable.  Delete
 	   # the old variable first.
 	   variable_delete ($varname) if $varname eq $var->name;
-	   # Define for all conditions.  Make sure we define
-	   # an empty variable in condition TRUE otherwise.
+	   # Define an empty variable in condition TRUE if there is no
+	   # result.
 	   @allresults = ([TRUE, '']) unless @allresults;
+	   # Define the rewritten variable in all conditions not
+	   # already covered by user definitions.
 	   foreach my $pair (@allresults)
 	     {
 	       my ($cond, @result) = @$pair;
-	       define ($varname, VAR_AUTOMAKE, '', $cond, "@result",
-		       '', $where, VAR_PRETTY)
-		 unless vardef ($varname, $cond);
-	       rvardef ($varname, $cond)->set_seen;
+	       my $var = var $varname;
+	       my @conds = ($var
+			    ? $var->not_always_defined_in_cond ($cond)->conds
+			    : $cond);
+
+	       foreach (@conds)
+		 {
+		   define ($varname, VAR_AUTOMAKE, '', $_, "@result",
+			   '', $where, VAR_PRETTY);
+		 }
 	     }
 	 }
+       set_seen $varname;
        return "\$($varname)";
      },
      %options);
